import json
import os
from saberlist.beatleaderAPI import BeatLeaderAPI
from collections import defaultdict
from datetime import datetime

HISTORY_FILE = "playlist_history.json"

def load_history():
    if os.path.exists(HISTORY_FILE):
        with open(HISTORY_FILE, 'r') as f:
            return json.load(f)
    return {}

def save_history(history):
    with open(HISTORY_FILE, 'w') as f:
        json.dump(history, f, indent=2)

def build_difficulty_based_playlist(api, player_id, history, playlist_name):
    scores_data = api.get_player_scores(player_id, use_cache=True)
    all_scores = scores_data['data']
    all_scores.sort(key=lambda x: x['timepost'])
    
    difficulty_groups = defaultdict(list)
    for score in all_scores:
        stars = score['leaderboard']['difficulty']['stars']
        song_id = score['leaderboard']['song']['id']
        difficulty_name = score['leaderboard']['difficulty']['difficultyName']
        
        # Check if this song:difficulty combination is in history
        if song_id not in history or difficulty_name not in history[song_id]:
            if 0 <= stars <= 3:
                difficulty_groups[0].append(score)
            elif 4 <= stars < 6:
                difficulty_groups[1].append(score)
            elif 6 <= stars < 7:
                difficulty_groups[2].append(score)
            elif stars >= 7:
                difficulty_groups[3].append(score)
    
    playlist_scores = []
    for difficulty, count in [(0, 5), (1, 5), (2, 5), (3, 5)]:
        unique_songs = {}
        for score in difficulty_groups[difficulty]:
            song_id = score['leaderboard']['song']['id']
            if song_id not in unique_songs or score['timepost'] < unique_songs[song_id]['timepost']:
                unique_songs[song_id] = score
        
        playlist_scores.extend(sorted(unique_songs.values(), key=lambda x: x['timepost'])[:count])
    
    # Update history
    for score in playlist_scores:
        song_id = score['leaderboard']['song']['id']
        difficulty_name = score['leaderboard']['difficulty']['difficultyName']
        if song_id not in history:
            history[song_id] = []
        history[song_id].append(difficulty_name)
    
    playlist_file = api.create_bplist(playlist_scores, playlist_name)
    
    return playlist_file, playlist_scores

def star_ladder():
    api = BeatLeaderAPI()
    player_id = '76561199407393962'
    
    history = load_history()
    
    # Generate a unique playlist name
    timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
    playlist_name = f"star_ladder-{timestamp}"
    
    playlist_file, playlist_scores = build_difficulty_based_playlist(api, player_id, history, playlist_name)
    save_history(history)

    print(f"Playlist created: {playlist_file}")
    print("Playlist contents:")
    for i, score in enumerate(playlist_scores, 1):
        song = score['leaderboard']['song']
        difficulty = score['leaderboard']['difficulty']
        print(f"{i}. {song['name']} by {song['author']} (Mapper: {song['mapper']}) - {difficulty['stars']:.2f} stars - Last played: {datetime.fromtimestamp(score['timepost'])}")

if __name__ == "__main__":
    star_ladder()